<?php

namespace Drupal\botlog;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides helper methods for botlog module.
 */
class BotlogHelper {
  use StringTranslationTrait;

  /**
   * The warning status level.
   *
   * @var int
   */
  public const STATUS_BAN = 3;

  /**
   * The warning status level.
   *
   * @var int
   */
  public const STATUS_ERROR = 2;

  /**
   * The white-list status level.
   *
   * @var int
   */
  public const STATUS_APPROVE = 0;

  /**
   * The warning status level.
   *
   * @var int
   */
  public const STATUS_WARN = 1;

  /**
   * The module name.
   *
   * @var string
   */
  protected const MODULE = 'botlog';

  /**
   * The configuration manager.
   *
   * @var Drupal\Core\Config\ConfigFactory
   */
  protected $cfg;

  /**
   * The current request.
   *
   * @var Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The SQL.
   *
   * @var Drupal\botlog\BotlogSql
   */
  protected $sql;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Config\ConfigFactory $cfg
   *   The configuration manager.
   * @param Drupal\botlog\BotlogSql $sql
   *   The current request.
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   */
  public function __construct(ConfigFactory $cfg, BotlogSql $sql, RequestStack $request) {
    $this->cfg = $cfg;
    $this->request = $request->getCurrentRequest();
    $this->sql = $sql;
  }

  /**
   * Insert an IP with error status.
   *
   * @param string $ip
   *   The IP.
   * @param mixed $data
   *   The data dump.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function banIp($ip, $data) {
    $fields = [
      'timestamp' => time(),
      'ip'        => $ip,
      'level'     => static::STATUS_BAN,
      'data'      => print_r($data, TRUE),
    ];
    $this->deleteByIp($ip);

    return $this->sql->insertIp($fields);
  }

  /**
   * Delete all entries with the supplied ID.
   *
   * @param string $ip
   *   The IP.
   *
   * @return int
   *   The Number of rows deleted.
   */
  public function deleteByIp($ip) {
    return $this->sql->deleteByIp($ip);
  }

  /**
   * Delete all entries older than the UNIX timestamp.
   *
   * @param string $timestamp
   *   The UNIX timestamp.
   *
   * @return int
   *   The Number of rows deleted.
   */
  public function deleteByTime($timestamp) {
    return $this->sql->deleteByTime($timestamp);
  }

  /**
   * Insert an IP with error status.
   *
   * @param string $ip
   *   The IP.
   * @param mixed $data
   *   The data dump.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function errorIp($ip, $data) {
    $fields = [
      'timestamp' => time(),
      'ip'        => $ip,
      'level'     => static::STATUS_ERROR,
      'data'      => print_r($data, TRUE),
    ];
    return $this->sql->insertIp($fields);
  }

  /**
   * Validate that this is a full IP, (4 octects).
   *
   * @param string $ip
   *   The IP.
   *
   * @return bool
   *   True if the string has a valid IP format.
   */
  public function fullIp($ip) {
    return preg_match('/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/', trim($ip));
  }

  /**
   * Configuration get wrapper.
   *
   * @param string $key
   *   The field name.
   * @param bool $reset
   *   Flush config cache.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function getCfg($key = '', $reset = FALSE) {
    $id = $this->getModuleName() . '.config';

    if ($reset) {
      $this->cfg->reset($id);
    }
    $cfg = $this->cfg->get($id);

    return $key ? $cfg->get($key) : (object) $cfg->get();
  }

  /**
   * Fetch the row count.
   *
   * @return int
   *   The number of rows in the botlog table.
   */
  public function getCount() {
    return $this->sql->getCount();
  }

  /**
   * Get the status of the IP.
   *
   * @param string $ip
   *   The IP.
   *
   * @return bool
   *   The block/no-block status.
   */
  public function getErrorStatus($ip) {
    return $this->sql->ipHasLevel($ip, static::STATUS_ERROR) ? TRUE : FALSE;
  }

  /**
   * Public access to the FACETS.
   *
   * @return array
   *   The FACETS.
   */
  public function getIp() {
    return $this->request->getClientIp();
  }

  /**
   * Fetch the row count.
   *
   * @param int $ip
   *   The IP.
   *
   * @return int
   *   The number of rows in the botlog table for the IP.
   */
  public function getIpCount($ip) {
    return $this->sql->getIpCount($ip);
  }

  /**
   * Get the level's display label.
   *
   * @param int $level
   *   The event level.
   *
   * @return string
   *   The level's display label.
   */
  public function getLevelLabel($level) {
    $levels = $this->getLevels();

    return !empty($levels[$level]) ? $levels[$level]->label : '';
  }

  /**
   * Public access to the FACETS.
   *
   * @param int $level
   *   The event level.
   *
   * @return array
   *   A level to label hash.
   */
  public function getLevels($level = '') {
    $levels = [
      static::STATUS_APPROVE => (object) [
        'label'       => $this->t('White-listed'),
        'description' => $this->t('Permanently white-listed.'),
        'constant'    => 'STATUS_APPROVE',
      ],
      static::STATUS_WARN    => (object) [
        'label'       => $this->t('Warning'),
        'description' => $this->t('Medium level event.'),
        'constant'    => 'STATUS_WARN',
      ],
      static::STATUS_ERROR   => (object) [
        'label'       => $this->t('Temporary Ban'),
        'description' => $this->t('High level event.'),
        'constant'    => 'STATUS_ERROR',
      ],
      static::STATUS_BAN     => (object) [
        'label'       => $this->t('Permanent Ban'),
        'description' => $this->t('Permanently black-listed.'),
        'constant'    => 'STATUS_BAN',
      ],
    ];
    if (!$level) {
      return $levels;
    }
    if ($level && !empty($levels[$level])) {
      return $levels[$level];
    }
    if ($level) {
      return '-';
    }
  }

  /**
   * Get the module's name.
   *
   * @return string
   *   The module's name.
   */
  public function getModuleName() {
    return static::MODULE;
  }

  /**
   * Get the status of the IP.
   *
   * @param string $ip
   *   The IP.
   *
   * @return bool
   *   The block/no-block status.
   */
  public function getStatus($ip) {
    return $this->getErrorStatus($ip);
  }

  /**
   * Get the white-listed of the IP.
   *
   * @return bool
   *   True if white-listed.
   */
  public function isWhiteListed() {
    $ip = $this->getIp() ? $this->getIp() : '';

    return ($ip && $this->sql->ipHasLevel($ip, static::STATUS_APPROVE)) ? TRUE : FALSE;
  }

  /**
   * Validate a partial IP.
   *
   * @param string $ip
   *   The IP.
   *
   * @return bool
   *   True if the string has a valid IP format.
   */
  public function partialIp($ip) {
    return preg_match('/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|\d{1,3}\.\d{1,3}\.\d{1,3}|\d{1,3}\.\d{1,3}|\d{1,3}$/', trim($ip));
  }

  /**
   * Redirect the bot if we have an error code.
   */
  public function redirectBadBot($tgt = '/500.html') {
    if (!$this->getErrorStatus($this->getIp())) {
      return;
    }
    $response = new RedirectResponse($tgt);
    $response->send();
  }

  /**
   * Configuration get wrapper.
   *
   * @param string $key
   *   The field name.
   * @param mixed $val
   *   The field value.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function setCfg($key = '', $val = NULL) {
    $id = $this->getModuleName() . '.config';

    $cfg = $this->cfg->getEditable($id);

    return $key ? $cfg->set($key, $val)->save() : NULL;
  }

  /**
   * Insert an IP with warning status.
   *
   * @param string $ip
   *   The IP.
   * @param mixed $data
   *   The data dump.
   *
   * @return object
   *   A new InsertQuery object for this connection.
   */
  public function warnIp($ip, $data) {
    $fields = [
      'timestamp' => time(),
      'ip'        => $ip,
      'level'     => static::STATUS_WARN,
      'data'      => print_r($data, TRUE),
    ];
    return $this->sql->insertIp($fields);
  }

  /**
   * White-list an IP.
   *
   * @param string $ip
   *   The IP.
   * @param mixed $data
   *   The data dump.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function whiteListIp($ip, $data) {
    $fields = [
      'timestamp' => time(),
      'ip'        => $ip,
      'level'     => static::STATUS_APPROVE,
      'data'      => print_r($data, TRUE),
    ];
    $this->deleteByIp($ip);

    return $this->sql->insertIp($fields);
  }

}
