<?php

namespace Drupal\botlog\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Url;
use Drupal\botlog\BotlogSql;
use Drupal\botlog\BotlogHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * IP search form.
 */
class BotlogForm extends FormBase {

  /**
   * Results per page.
   *
   * @var int
   */
  protected const RPP = 50;

  /**
   * Report helper.
   *
   * @var Drupal\botlog\BotlogHelper
   */
  protected $helper;

  /**
   * The page manager.
   *
   * @var Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerMgr;

  /**
   * Redirect.
   *
   * @var Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $redirect;

  /**
   * Report SQL.
   *
   * @var Drupal\botlog\BotlogSql
   */
  protected $sql;

  /**
   * Class constructor.
   *
   * @param Drupal\Core\Pager\PagerManagerInterface $pager_mgr
   *   The language manager dependency injection.
   * @param Drupal\botlog\BotlogHelper $helper
   *   The report helper.
   * @param Drupal\botlog\BotlogSql $sql
   *   The reports SQL.
   * @param Drupal\Core\Routing\RedirectDestinationInterface $redirect
   *   The redirect manager.
   */
  public function __construct(PagerManagerInterface $pager_mgr, BotlogHelper $helper, BotlogSql $sql, RedirectDestinationInterface $redirect) {
    $this->helper = $helper;
    $this->pagerMgr = $pager_mgr;
    $this->redirect = $redirect;
    $this->sql = $sql;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $c) {
    return new static($c->get('pager.manager'), $c->get('botlog.helper'), $c->get('botlog.sql'), $c->get('redirect.destination'));
  }

  /**
   * The form ID.
   *
   * @return string
   *   The form ID.
   */
  public function getFormId() {
    return 'botlog_search';
  }

  /**
   * The bot list.
   *
   * @param array $form
   *   A drupal form array.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   A Drupal form state object.
   * @param string $ip
   *   The IP.
   *
   * @return array
   *   A Drupal form array.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $ip = '') {
    $rows = [];
    $header = [
      $this->t('Time'),
      $this->t('IP'),
      $this->t('Status'),
      $this->t('Details'),
      $this->t('Delete'),
    ];
    if ($ip) {
      $pattern = $this->sql->escapeLike($ip);
      $page = $this->pagerMgr->createPager($this->sql->getCount($pattern), self::RPP);
      $events = $this->sql->search($pattern, ($page->getCurrentPage() * self::RPP), self::RPP);
    }
    else {
      $page = $this->pagerMgr->createPager($this->sql->getCount(), self::RPP);
      $events = $this->sql->selectAllEvents(($page->getCurrentPage() * self::RPP), self::RPP);
    }
    foreach ($events as $obj) {
      $rows[$obj->bid] = [
        $obj->timestamp ? date('Y-m-d H:i:s', $obj->timestamp) : '-',
        $obj->ip ? $obj->ip : '-',
        $this->getLevelTxt($obj),
        $this->getDetailLink($obj),
        $this->getDeleteLink($obj),
      ];
    }
    $form['pager_head'] = [
      '#type' => 'pager',
    ];
    $form['ip'] = [
      '#type'          => 'textfield',
      '#default_value' => $ip,
    ];
    $form['search'] = [
      '#type'   => 'submit',
      '#name'   => 'search',
      '#value'  => $this->t('Search'),
      '#submit' => ['::submitForm'],
    ];
    $form['list'] = [
      '#type'   => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#empty'  => $this->t('Nothing found.'),
    ];
    $form['pager_foot'] = [
      '#type' => 'pager',
    ];
    return $form;
  }

  /**
   * Custom submit.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirectUrl(Url::fromUri('internal:/admin/reports/botlog/monitor/' . $form_state->getValue('ip')));
  }

  /**
   * Get the level text label.
   *
   * @param object $obj
   *   The event object.
   *
   * @return string
   *   The level text label.
   */
  protected function getLevelTxt($obj) {
    return $this->helper->getLevelLabel($obj->level);
  }

  /**
   * Get the delete link.
   *
   * @param object $obj
   *   The event object.
   *
   * @return string
   *   A link to the delete page.
   */
  protected function getDeleteLink($obj) {
    return !empty($obj->bid) ? Link::fromTextAndUrl($this->t('Delete'), Url::fromUri("internal:/admin/reports/botlog/delete/$obj->bid")) : '-';
  }

  /**
   * Get the detail link.
   *
   * @param object $obj
   *   The event object.
   *
   * @return string
   *   A link to the detail page.
   */
  protected function getDetailLink($obj) {
    return !empty($obj->bid) ? Link::fromTextAndUrl($this->t('Details'), Url::fromUri("internal:/admin/reports/botlog/detail/$obj->bid")) : '-';
  }

}
